-------------------------------------------------------------------------------- 
-- UPC - EETAC - EEL - CSD - DIGSYS       
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
-- Example of a complete Timer_MMSS
-- CLK_Generator 
-- Strategy: Schematic using frequency dividers and TFF to square signals
---------------------------------------------------
-- Project P8  
-- http://digsys.upc.edu
--------------------------------------------------------------------------------
-- OSC_CLK_in = 50 MHz  (Xilinx NEXYS 2 board)
-- CLK_1Hz_SQ, to move the Datapath's couunter in seconds 
-- CLK_20Hz_SQ, to move the Datapath's couunter to set time Up or Down faster
-- CLK_1_6kHz_SQ, to generate an audible sound wave 

-- CLK_100kHz_SQ, to run the "system CLK" of the Control Unit  

--------------------------------------------------------------------------------
LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
	
ENTITY CLK_Generator IS  
      PORT(	
			OSC_CLK_in		:	IN		STD_LOGIC;
			CD 				:	IN		STD_LOGIC;
			CE					:	IN		STD_LOGIC;
			
			CLK_1Hz_SQ		:	OUT	STD_LOGIC;
			CLK_20Hz_SQ		:	OUT	STD_LOGIC;
			CLK_1_6kHz_SQ	:	OUT	STD_LOGIC;
			CLK_100kHz_SQ	:	OUT	STD_LOGIC	
		);
END CLK_Generator;

ARCHITECTURE schematic OF CLK_Generator  IS  
-- Components
	COMPONENT Freq_Div_125	IS                 
		PORT(	
			CD,CLK,CE	: 	IN  std_logic;
			TC125		:	OUT std_logic
		);
	END COMPONENT;	

	COMPONENT Freq_Div_80 IS                 
		PORT(
			CD,CLK,CE	: 	IN  std_logic;
			TC80		:	OUT std_logic
		);
	END COMPONENT;	

	COMPONENT Freq_Div_20 IS                 
		PORT(	
			CD,CLK,CE	: 	IN  std_logic;
			TC20		:	OUT std_logic
		);
	END COMPONENT;	

	COMPONENT Freq_Div_2 IS                 
		PORT(	
			CD,CLK,CE	: 	IN  std_logic;
			TC2		:	OUT std_logic
		);
	END COMPONENT;	
	
	COMPONENT T_Flip_Flop IS
		Port (	
			CLK 	: IN	STD_LOGIC;
			CD 		: IN	STD_LOGIC;
			T 		: IN	STD_LOGIC;
			Q 		: OUT	STD_LOGIC
		);
	END COMPONENT;					

-- Signals for connecting components together (just the internal wires)
	SIGNAL CLK				:	std_logic;
	SIGNAL Pulsed_400kHz	:	std_logic;
	SIGNAL Pulsed_200kHz	:	std_logic;
	SIGNAL Pulsed_3_2kHz	:	std_logic;	
	SIGNAL Pulsed_40Hz	:	std_logic;	
	SIGNAL Pulsed_2Hz		:	std_logic;
		
	BEGIN  
-- Instantiation of components

	Chip1	: 	Freq_Div_125
	PORT MAP (	   	
-- from component name 	=> to signal or port name
			CLK		=> CLK,
			CD 		=> CD,
			CE 		=> CE,
			TC125	=> Pulsed_400kHz					  
		);

	Chip2	:  Freq_Div_125
		PORT MAP (
-- from component name 	=> to signal or port name
			CLK		=> CLK,
			CD 		=> CD,
			CE 		=> Pulsed_400kHz,
			TC125 	=> Pulsed_3_2kHz					  
		);

	Chip3	: 	Freq_Div_80
		PORT MAP (	   	
-- from component name 	=> to signal or port name
			CLK		=> CLK,
			CD 		=> CD,
			CE 		=> Pulsed_3_2kHz,
			TC80		=> Pulsed_40Hz					  
		);

	Chip4	:  Freq_Div_20
		PORT MAP (
-- from component name 	=> to signal or port name
			CLK		=> CLK,
			CD 		=> CD,
			CE 		=> Pulsed_40Hz,
			TC20 	=> Pulsed_2Hz					  
		);
	
	Chip5	:  Freq_Div_2
		PORT MAP (
-- from component name 	=> to signal or port name
			CLK		=> CLK,
			CD 		=> CD,
			CE 		=> Pulsed_400kHz,
			TC2 	=> Pulsed_200kHz					  
		);
	
	Chip6	: T_Flip_Flop
		PORT MAP (
	-- from component name 	=> to signal or port name	
			CLK		=> CLK,
			CD 		=> CD,
			T		=> Pulsed_200kHz,
			Q 		=> CLK_100kHz_SQ				  
		);		

	Chip7	: T_Flip_Flop
		PORT MAP (
	-- from component name 	=> to signal or port name	
			CLK		=> CLK,
			CD 		=> CD,
			T		=> Pulsed_3_2kHz,
			Q 		=> CLK_1_6kHz_SQ				  
		);		

	Chip8	: T_Flip_Flop
		PORT MAP (
	-- from component name 	=> to signal or port name	
			CLK		=> CLK,
			CD 		=> CD,
			T		=> Pulsed_40Hz,
			Q 		=> CLK_20Hz_SQ				  
		);		


	Chip9	: T_Flip_Flop
		PORT MAP (
	-- from component name 	=> to signal or port name	
			CLK		=> CLK,
			CD 		=> CD,
			T		=> Pulsed_2Hz,
			Q 		=> CLK_1Hz_SQ				
		);		

	
			
-- Connections and logic between components

	-- The internal CLK form the external quartz crystal 
	CLK <= OSC_CLK_in;


	
END schematic ;
