--------------------------------------------------------------------------------
-- Example of a complete Timer_MMSS
-- The control unit component 
-- Strategy: synchronous FSM 
---------------------------------------------------
-- Project P8  - CSD 
-- http://digsys.upc.edu
--------------------------------------------------------------------------------
LIBRARY ieee;
USE  IEEE.STD_LOGIC_1164.all;
USE  IEEE.STD_LOGIC_ARITH.all;
USE  IEEE.STD_LOGIC_UNSIGNED.all;

			
ENTITY Control_unit IS
 		Port (	
  	    	CLK 		:	IN	STD_LOGIC;
			CD   		:	IN	STD_LOGIC;
			
			Start   	:	IN	STD_LOGIC;  -- external control buttons
			Stop		:	IN	STD_LOGIC;
			ST_up		:	IN		STD_LOGIC;
			ST_down	:	IN		STD_LOGIC;

			TCH		:	IN	 std_logic;  -- from the datapath	
			
			CE				: 	OUT  std_logic; --to the datapath
			UD_L			: 	OUT  std_logic;
			SEL_CLK		: 	OUT  std_logic;
			SEL_sound	: 	OUT  std_logic;
			SEL_sec_LED	: 	OUT  std_logic;	
			
			Alarm_LED	: 	OUT  STD_LOGIC;	-- outputs  
			Timer_LED	: 	OUT  STD_LOGIC				
		);

END Control_unit;


ARCHITECTURE FSM_like OF Control_unit IS


	TYPE State_type IS (Idle, Set_Time_Up, Set_Time_Down, Timing, Alarm_Set);  
	SIGNAL current_state, next_state	: State_type ; 	
------------------------------------------------------------------------------
-- Xilinx (click the ISE tap: "Design goals & strategies", file:default.xds)  
-- The legal values for this are "one_hot", "sequential", "johnson", 
-- "gray", and "auto".

--	ATTRIBUTE fsm_encoding : STRING;
--	ATTRIBUTE fsm_encoding of State_type : TYPE IS "johnson";
--
-- FSM encoding for Lattice 
-- ATTRIBUTE syn_encoding : STRING;
-- ATTRIBUTE syn_encoding OF State_type : TYPE IS "one-hot";


BEGIN

	-------- The state register that contains the memory cells
state_register: PROCESS (CD, CLK, next_state)
	BEGIN
						
	IF CD = '1' THEN 						
		current_state <= Idle; -- reset state where to go if a CD signal is asserted 
	ELSIF (CLK'EVENT and CLK = '1') THEN	-- Synchronous D-type flip-flops
		current_state <= next_state; 
	END IF;

END PROCESS state_register;

	-------- CC1: Combinational system for calculating next state
	-------- The truth table that contains all the state transitions (arrows)

CC_1: PROCESS (current_state, ST_Up, ST_Down, TCH, Start, Stop)
	BEGIN
	   
		CASE current_state IS
				
			WHEN Idle =>
			   IF (ST_up = '1' and ST_down = '0') THEN
					next_state <= Set_Time_Up;	
				ELSIF (ST_up = '0' and ST_down = '1') THEN
					next_state <= Set_Time_Down;		
				ELSIF (Start = '1' and ST_down = '0' and ST_up = '0') THEN
					next_state <= Timing;	
				ELSE    
					next_state <= Idle; 
				END IF;
				
			WHEN Set_Time_Up =>  
				IF (ST_up = '1') THEN
					next_state <= Set_Time_Up; 
				ELSE    
					next_state <= Idle ;
				END IF;
				
			WHEN Set_Time_Down => 
				IF (ST_down = '1') THEN
					next_state <= Set_Time_Down; 
				ELSE    
					next_state <= Idle ;
				END IF;
				
			WHEN Timing => 
				IF (TCH = '1') THEN
					next_state <= Alarm_Set; 
-- 				ELSIF (Stop = '1') THEN
	-- 				next_state <= Idle;	-- This stops de timing like a synchronous CD	 
				ELSE    
					next_state <= Timing;
				END IF;
				
			WHEN Alarm_Set => 
			   IF (Stop = '1' ) THEN
			      next_state <= Idle; 
				ELSE    
					next_state <= Alarm_Set ;
				END IF;
				
		END CASE ;		

END PROCESS CC_1;

	-------- CS_2: combinational system for calculating the outputs 
CC_2: PROCESS (current_state)
	BEGIN
		CASE current_state IS
			WHEN Idle => 
				CE 			<= '0'; -- Disable the counter
				UD_L			<= '0';		
				SEL_CLK		<= '0';			
				SEL_sound	<= '0';		-- No sound  
				SEL_sec_LED	<= '0';	-- No second indication
				
				Alarm_LED	<= '0';	-- No Alarm LED	
				Timer_LED	<= '0';	-- Timer not running		

			WHEN Set_Time_Up => 
				CE 			<= '1';   -- Emable the counter
				UD_L			<= '1';	 -- Counting Up fast	
				SEL_CLK		<= '1';	 -- 20 Hz		
				SEL_sound	<= '0';		
				SEL_sec_LED	<= '0';	
				
				Alarm_LED	<= '0';		
				Timer_LED	<= '0';	

			WHEN Set_Time_Down => 
				CE 			<= '1';   -- Emable the counter
				UD_L			<= '0';	 -- Counting Down fast	
				SEL_CLK		<= '1';	 -- 20 Hz	
				SEL_sound	<= '0';		
				SEL_sec_LED	<= '0';	
				
				Alarm_LED	<= '0';		
				Timer_LED	<= '0';		

			WHEN Timing => 
				CE 			<= '1';   -- Emable the counter
				UD_L			<= '0';	 -- Counting Down when timing	
				SEL_CLK		<= '0';	 -- 1 Hz		
				SEL_sound	<= '0';		
				SEL_sec_LED	<= '1';	-- Indication of the seconds ticking
				
				Alarm_LED	<= '0';		
				Timer_LED	<= '1';	-- Indication of timing

			WHEN Alarm_Set => 
				CE 			<= '0';   -- Disable the counter
				UD_L			<= '0';	 -- don't care
				SEL_CLK		<= '0';	 -- 		
				SEL_sound	<= '1';	 -- Generate the audible soundwave	
				SEL_sec_LED	<= '0';	 
				
				Alarm_LED	<= '1';	-- Indication of Alarm LED	
				Timer_LED	<= '0';	-- Indication of timing

		END CASE ;		

END PROCESS CC_2;

-- Place here other logic if necessary

END FSM_like;