--------------------------------------------------------------------------------
-- An example of a synchronous universal binary counter  
-- FSM style 
-- This component may be used as a building block for more complex circuits
---------------------------------------------------
-- Project P7  - CSD 
-- http://digsys.upc.edu
--------------------------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;


ENTITY  Counter_mod4 is

	port(	CE 	:	in std_logic;
			CD 	:	in std_logic;
			CLK	:	in std_logic;
			TC4	:	out std_logic; 
			Q	:	out std_logic_vector(1 downto 0)
			
		);
end Counter_mod4;

architecture FSM_style of Counter_mod4 is

	TYPE State_type IS (Num0, Num1, Num2, Num3);  

	SIGNAL present_state, future_state	: State_type ; 
	
-- Constants for special states (the first and the last state)
CONSTANT Reset		: State_type := Num0; -- The fisrt state. This is another name for the state Num0
CONSTANT Max_count	: State_type := Num3; -- The last state. This is anather way to name the state Num9


begin   

---------------------- State Register: the only clocked block.  
---------------------- The "memory" of the system (future events will depend on past events 

state_register: PROCESS (CD,CLK, future_state)
	BEGIN
						
	IF CD = '1' THEN 			-- reset counter
		present_state <= Reset; 
	ELSIF (CLK'EVENT and CLK = '1') THEN	-- Synchronous register (D-type flip-flop)
		present_state <= future_state; 
	END IF;

END PROCESS state_register;

------------------------- CC1: Combinational system for calculating next state

CC_1: PROCESS (present_state, CE)
	BEGIN
		IF CE = '0' THEN
			future_state <= present_state;  -- count disable		

		ELSE					-- just a simple state up count
			CASE present_state IS
				WHEN Num0 => 
					future_state <= Num1 ;
				WHEN Num1 => 
					future_state <= Num2 ;
				WHEN Num2 => 
					future_state <= Num3 ;
				WHEN Num3 => 
					future_state <= Num0 ;
			END CASE ;		

		END IF;

END PROCESS CC_1;

------------------------- CS_2: combinational system for calculating extra outputs 
-------------------------- and outputing the present state (the actual count)
		
CC_2: PROCESS (present_state, CE)
	BEGIN
-- The terminal count output
		IF ((present_state = Max_count) AND (CE = '1') ) THEN 
			TC4 <= '1';
		ELSE 	
			TC4 <= '0'; 
		END IF;


-- And now just copying the present state to the output:
		CASE present_state IS
				WHEN Num0 => 
					Q <= "00";
				WHEN Num1 => 
					Q <= "01";
				WHEN Num2 => 
					Q <= "10";
				WHEN Num3 => 
					Q <= "11";
		END CASE ;		


END PROCESS CC_2;

		
end FSM_style;