-------------------------------------------------------------------------------- 
-- UPC - EETAC - EEL - CSD - DIGSYS http://digsys.upc.edu     
--------------------------------------------------------------------------------
-- P8.  
-- Multiplexed 7-segment display system 
--------------------------------------------------------------------------------
LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
	
ENTITY MDS IS  
      PORT(	
		Unit, Tens					: 	in std_logic_vector(3 downto 0);
		Hundreds, Thousands			: 	in std_logic_vector(3 downto 0);
		DP_units, DP_tens			: 	in std_logic; 
		DP_hundreds, DP_thousands	: 	in std_logic; 
		A_L, B_L, C_L, D_L 			: 	OUT STD_LOGIC; 
		E_L, F_L, G_L  				: 	OUT STD_LOGIC;    
		CA_L						:	OUT std_logic_vector (3 downto 0);
		DP_L						:	OUT std_logic;   
		CLK							: 	IN std_logic
	);
END MDS ;

ARCHITECTURE schematic OF MDS  IS  
-- Components
	COMPONENT Quad_MUX4 IS                 
		port(	
			E 	:	in std_logic;
			Ch0	:	in std_logic_vector(3 downto 0);
			Ch1	:	in std_logic_vector(3 downto 0);
			Ch2	:	in std_logic_vector(3 downto 0);
			Ch3	:	in std_logic_vector(3 downto 0);
			s	:	in std_logic_vector(1 downto 0);
			y	:	out std_logic_vector(3 downto 0)
		);
	END COMPONENT;	

	COMPONENT Counter_mod4 IS                 
	port(	CE 	:	in std_logic;
			CD 	:	in std_logic;
			CLK	:	in std_logic;
			TC4	:	out std_logic; 
			Q	:	out std_logic_vector(1 downto 0)
			
		);
	END COMPONENT;	

	COMPONENT Mux_4 IS                 
	port(	E 	:	in std_logic;
			Ch0	:	in std_logic;
			Ch1	:	in std_logic;
			Ch2	:	in std_logic;
			Ch3	:	in std_logic;
			s	:	in std_logic_vector(1 downto 0);
			y	:	out std_logic
		);
	END COMPONENT;	
 
	COMPONENT Dec_2_to_4 IS                 
		port(	
			E 			:	in std_logic;
			X_in		:	in std_logic_vector (1 downto 0);
			Yout_L	:	out std_logic_vector(3 downto 0)
		);
	END COMPONENT;	
	  		  	  
	COMPONENT HEX_7SEG_DECODER IS
 		PORT (
			X_IN						            	: IN STD_LOGIC_VECTOR(3 DOWNTO 0); 
			BI_L	                    	        	: IN STD_LOGIC;      
			A_L, B_L, C_L, D_L, E_L, F_L, G_L  			: OUT STD_LOGIC
			);
	END COMPONENT;					

-- Signals for connecting components together (just the internal wires)
	SIGNAL SEL		:	std_logic_vector (1 downto 0);
	SIGNAL DIG		:	std_logic_vector (3 downto 0);
	SIGNAL TC4, CD	:	std_logic; 
	SIGNAL DPTH_L, DPH_L, DPT_L, DPU_L	:	std_logic; 
		
	BEGIN  
-- Instantiation of components

	CHIP1	: 	HEX_7SEG_DECODER
	PORT MAP (	   	
-- from component name 	=> to signal or port name
			X_IN		=> DIG,
			BI_L 		=> '1',
			A_L 		=> A_L,
			B_L		=> B_L,
			C_L 		=> C_L,
			D_L		=> D_L,
			E_L 		=> E_L,
			F_L		=> F_L,
			G_L 		=> G_L				   				  
		);

	CHIP2	:  Mux_4
		PORT MAP (
-- from component name 	=> to signal or port name
			E		=> '1',
			Ch0 	=> DPU_L,
			Ch1 	=> DPT_L,
			Ch2 	=> DPH_L, 
			Ch3 	=> DPTH_L,
			s		=> SEL,
			y		=> DP_L
		);
							

	CHIP3	: 	Quad_MUX4
		PORT MAP (
-- from component name 	=> to signal or port name
			E		=> '1',
			Ch0 	=> Unit,
			Ch1 	=> Tens,
			Ch2 	=> Hundreds,
			Ch3 	=> Thousands, 
			s		=> SEL,
			y		=> DIG				  
		);

	CHIP4	:	Counter_mod4
		PORT MAP (
-- from component name 	=> to signal or port name
			CD 	=> '0',
			TC4	=> TC4,
			CE		=> '1',
	   	CLK	=> CLK,
			Q		=> SEL
		);

	Chip5	: Dec_2_to_4
		PORT MAP (
	-- from component name 	=> to signal or port name	
			X_in		=> SEL,
			E 			=> '1',
			Yout_L	=> CA_L
		);		
			
-- connections and logic between components
	
DPU_L <= NOT(DP_units);
DPT_L <= NOT(DP_Tens);
DPH_L <= NOT(DP_Hundreds); 
DPTH_L <= NOT(DP_Thousands);

			
	
END schematic ;
