--------------------------------------------------------------------------------
-- Example of a complete Timer_MMSS
-- Strategy: Dedicated processor with CLK_Generator
---------------------------------------------------
-- Project P8  - CSD 
-- http://digsys.upc.edu
--------------------------------------------------------------------------------

LIBRARY IEEE;
   USE IEEE.STD_LOGIC_1164.ALL;
	
ENTITY Timer_MMSS IS  
	PORT(	
		CD					: 	IN  std_logic;
		OSC_CLK_in		: 	IN  std_logic;
		Start, Stop		: 	IN  std_logic;
		ST_up, ST_down	: 	IN  std_logic;
		DMT, DMU			: 	OUT STD_LOGIC_VECTOR(3 DOWNTO 0);	  
		DST, DSU			: 	OUT STD_LOGIC_VECTOR(3 DOWNTO 0);	  
		S_LED				: 	OUT STD_LOGIC;	  
		Alarm_LED		: 	OUT STD_LOGIC;	  
		Timer_LED		: 	OUT STD_LOGIC;	  
		Soundwave		: 	OUT STD_LOGIC;
		CLK_1_6kHz_SQ_out	: 	OUT STD_LOGIC -- extra output to drive the MDS 		
	);
END Timer_MMSS ;

ARCHITECTURE schematic OF Timer_MMSS  IS  
-- Components

	COMPONENT Datapath IS  
       PORT(
			CD					: 	IN std_logic;
			
			CLK_1Hz_SQ		: 	IN std_logic; --CLK's for the couter
			CLK_20Hz_SQ		: 	IN std_logic;
							
			CLK_1_6kHz_SQ	: 	IN std_logic; --CLK for audio sound
			
			CE					: 	IN std_logic; -- to control the Counter_MMSS
			UD_L				: 	IN std_logic;
			SEL_CLK			: 	IN std_logic;
			
			SEL_sound		: 	IN std_logic; -- Control the sound ON/OFF
			SEL_sec_LED		: 	IN std_logic; -- Control the Seconds LED	   

			DMT, DMU			: 	OUT STD_LOGIC_VECTOR(3 DOWNTO 0);	  
			DST, DSU			: 	OUT STD_LOGIC_VECTOR(3 DOWNTO 0);	
			S_LED				:	OUT std_logic;
			Soundwave		: 	OUT STD_LOGIC;			
			
			TCH				:	OUT std_logic  -- this is the status signal	
		);
	END COMPONENT;   
	
	COMPONENT Control_unit IS  
 		Port (	
  	    	CLK 			:	IN	STD_LOGIC;
			CD   			:	IN	STD_LOGIC;
			
			Start   		:	IN	STD_LOGIC;  -- external control buttons
			Stop			:	IN	STD_LOGIC;
			ST_up			:	IN	STD_LOGIC;
			ST_down		:	IN	STD_LOGIC;

			TCH			:	IN std_logic;  -- from the datapath	
			
			CE				: 	OUT  std_logic; --to the datapath
			UD_L			: 	OUT  std_logic;
			SEL_CLK		: 	OUT  std_logic;
			SEL_sound	: 	OUT  std_logic;
			SEL_sec_LED	: 	OUT  std_logic;	
			
			Alarm_LED	: 	OUT  STD_LOGIC;	-- outputs  
			Timer_LED	: 	OUT  STD_LOGIC				
		);
	END COMPONENT;        

	COMPONENT CLK_Generator IS  
 		Port (	
			OSC_CLK_in		:	IN	STD_LOGIC;
		   CD 				:	IN	STD_LOGIC;
		   CE					:	IN	STD_LOGIC;
			CLK_1Hz_SQ		:	OUT	STD_LOGIC;
			CLK_20Hz_SQ		:	OUT	STD_LOGIC;
			CLK_1_6kHz_SQ	:	OUT	STD_LOGIC;
			CLK_100kHz_SQ	:	OUT	STD_LOGIC	
		);
	END COMPONENT;  


-- Signals for connecting components together (the internal wires)
	SIGNAL CE, UD_L, SEL_CLK, SEL_sound, SEL_sec_LED	:	STD_LOGIC; 
	SIGNAL CLK_100kHz_SQ, CLK_20Hz_SQ					:	STD_LOGIC; 
	SIGNAL CLK_1_6kHz_SQ, CLK_1Hz_SQ					:	STD_LOGIC; 
	SIGNAL TC_Hours										:	STD_LOGIC; 
	
	BEGIN  
-- Instantiation of components

	Chip1	: 	Datapath
	PORT MAP (	   	
-- From component name 	=> 	to signal or port name
			CLK_1Hz_SQ		=> 	CLK_1Hz_SQ,
			CLK_1_6kHz_SQ	=> 	CLK_1_6kHz_SQ,
			CLK_20Hz_SQ		=> 	CLK_20Hz_SQ,			
		   CD 				=> 	CD,
			CE					=> 	CE, 
			UD_L				=> 	UD_L, 
			SEL_CLK			=> 	SEL_CLK, 
			SEL_sound		=> 	SEL_sound, 
			SEL_sec_LED		=> 	SEL_sec_LED, 

			DMT				=> 	DMT,
			DMU				=> 	DMU,
			DST				=> 	DST,
			DSU				=> 	DSU,

			S_LED				=> 	S_LED,
			Soundwave		=>		Soundwave,	
		
			TCH				=>	TC_Hours --terminal count hours
		);

	Chip2	: 	Control_unit
	PORT MAP (	   	
-- From component name 	=> 	to signal or port name
  	    	CLK			=> 	CLK_100kHz_SQ, --system CLK
			CD				=> 	CD,
			
			Start			=> 	Start,-- external buttons
			Stop			=> 	Stop,
			ST_up			=> 	ST_up,
			ST_down		=> 	ST_down,

			TCH			=> 	TC_Hours,-- status input from the datapath	
			
			CE				=> 	CE,-- internal wires to the datapath
			UD_L			=> 	UD_L,
			SEL_CLK		=> 	SEL_CLK,
			SEL_sound	=> 	SEL_sound,
			SEL_sec_LED	=> 	SEL_sec_LED,
			
			Alarm_LED	=> 	Alarm_LED,-- outputs 
			Timer_LED	=> 	Timer_LED	
		);

	Chip3	: 	CLK_Generator
	PORT MAP (	   	
-- From component name 		=> 	to signal or port name
			OSC_CLK_in		=> 	OSC_CLK_in,
	   	CD 				=> 	CD,
	   	CE					=> 	'1',
			CLK_1Hz_SQ		=>	CLK_1Hz_SQ,
			CLK_20Hz_SQ		=>	CLK_20Hz_SQ,
			CLK_1_6kHz_SQ	=>	CLK_1_6kHz_SQ,
			CLK_100kHz_SQ	=>	CLK_100kHz_SQ
	);
		

-- connections and logic between components
	 
CLK_1_6kHz_SQ_out <= CLK_1_6kHz_SQ;
	
END schematic ;
