--------------------------------------------------------------------------------
-- Example of a complete Timer_MMSS
-- Strategy: Dedicated processor with CLK_Generator
-- This schematic includes the multiplexed display system necessary to drive the 
-- NEXYS 2 board 4 digit 7-segment displays
---------------------------------------------------
-- Project P8  - CSD 
-- http://digsys.upc.edu
--------------------------------------------------------------------------------

LIBRARY IEEE;
   USE IEEE.STD_LOGIC_1164.ALL;


ENTITY Timer_MMSS_NEXYS2 IS  
	PORT(	
		CD					: 	IN  std_logic;
		OSC_CLK_in			: 	IN  std_logic;
		Start, Stop			: 	IN  std_logic;
		ST_up, ST_down		: 	IN  std_logic;
	
		A_L, B_L, C_L, D_L 	: 	OUT STD_LOGIC; 
		E_L, F_L, G_L  		: 	OUT STD_LOGIC;    
		DP_L				:	OUT std_logic; 
		CA_L				:	OUT std_logic_vector (3 downto 0);

		Alarm_LED			: 	OUT STD_LOGIC;	  
		Timer_LED			: 	OUT STD_LOGIC;	  
		Soundwave			: 	OUT STD_LOGIC 		
								
	);
END Timer_MMSS_NEXYS2 ;

ARCHITECTURE schematic OF Timer_MMSS_NEXYS2  IS  
-- Components
COMPONENT MDS IS  
      PORT(	
		Unit, Tens					: 	in std_logic_vector(3 downto 0);
		Hundreds, Thousands			: 	in std_logic_vector(3 downto 0);
		DP_units, DP_tens			: 	in std_logic; 
		DP_hundreds, DP_thousands	: 	in std_logic; 
		A_L, B_L, C_L, D_L 			: 	OUT STD_LOGIC; 
		E_L, F_L, G_L  				: 	OUT STD_LOGIC;    
		CA_L						:	OUT std_logic_vector (3 downto 0);
		DP_L						:	OUT std_logic;   
		CLK							: 	IN std_logic
	);
END COMPONENT;  

COMPONENT Timer_MMSS IS  
	PORT(	
		CD					: 	IN  std_logic;
		OSC_CLK_in			: 	IN  std_logic;
		Start, Stop			: 	IN  std_logic;
		ST_up, ST_down		: 	IN  std_logic;
		DMT, DMU			: 	OUT STD_LOGIC_VECTOR(3 DOWNTO 0);	  
		DST, DSU			: 	OUT STD_LOGIC_VECTOR(3 DOWNTO 0);	  
		S_LED				: 	OUT STD_LOGIC;	  
		Alarm_LED			: 	OUT STD_LOGIC;	  
		Timer_LED			: 	OUT STD_LOGIC;	  
		Soundwave			: 	OUT STD_LOGIC;
		CLK_1_6kHz_SQ_out	: 	OUT STD_LOGIC -- extra output to drive the MDS 
	);
END COMPONENT;  

-- Signals for connecting components together (the internal wires)

	SIGNAL	DMT, DMU			: 	STD_LOGIC_VECTOR(3 DOWNTO 0);	  
	SIGNAL	DST, DSU			: 	STD_LOGIC_VECTOR(3 DOWNTO 0);	  
	SIGNAL	S_LED				: 	STD_LOGIC;	  
	SIGNAL CLK_1_6kHz_SQ		:	STD_LOGIC; 
	

BEGIN  
-- Instantiation of components

	Chip1	: 	Timer_MMSS
	PORT MAP (	   	
-- From component name 		=> 	to signal or port name     
		CD		 			=> 	CD,		
		OSC_CLK_in			=> 	OSC_CLK_in,		
		Start				=> 	Start,		
		Stop				=> 	Stop,	
		ST_up				=> 	ST_up,	
		ST_down				=> 	ST_down,		
		DMT 				=>	DMT,
		DMU 				=>	DMU,			  
		DST 				=>	DST,
		DSU 				=>	DSU,			  
		S_LED	 			=> 	S_LED,		  
		Alarm_LED 			=> 	Alarm_LED,	  
		Timer_LED 			=> 	Timer_LED, 
		Soundwave 			=> 	Soundwave,				
		CLK_1_6kHz_SQ_out	=> 	CLK_1_6kHz_SQ	
	);

	Chip2	: 	MDS     -- The miltiplexed digit systne
	PORT MAP (	   	
-- From component name 	=> 	to signal or port name
		Thousands		=> DMT,
		Hundreds		=> DMU,
		Tens			=> DST,
		Unit	   		=> DSU,
		DP_thousands	=> '0',
		DP_hundreds		=> S_LED,		
		DP_tens			=> '0',	
		DP_units		=> '0',
		A_L				=> A_L,
		B_L				=> B_L,
		C_L	   			=> C_L,
		D_L				=> D_L, 			
		E_L				=> E_L,
		F_L				=> F_L,
		G_L				=> G_L,  			 
		DP_L	  		=> DP_L,					  
		CA_L	   		=> CA_L,
		CLK				=> CLK_1_6kHz_SQ							
	);

-- connections and logic between components
	
END schematic ;
